using System;
using System.Runtime.InteropServices;
using System.Text.RegularExpressions;
using System.IO;
using Gtk;
using Gdk;
using System.Diagnostics;

namespace FastbootGUI
{
    public partial class HelpWindow : Gtk.Window
    {
        private int state = 0;
        private int stateMAX = 5;
        private string os;
        private Gtk.Image pic;
        private MainWindow win;
		
		// Windows dll import
        [DllImport("libusb0.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int usb_install_driver_np(string inf_file);

        public HelpWindow(string os_, MainWindow win_) :
            base(Gtk.WindowType.Toplevel)
        {
            this.Build();
            os = os_;
            win = win_;
            if (Regex.IsMatch(os_, "Windows"))
            {
                frame1.Visible = false;
                BtnK4Driver.Visible = false;
                BtnKTDriver.Visible = false;

                pic = new Gtk.Image(Directory.GetCurrentDirectory() + "\\lib\\pics\\1.jpg");
            }
            else
            {
                pic = new Gtk.Image(Directory.GetCurrentDirectory() + "/lib/pics/1.jpg");
            }
            image2.Pixbuf = pic.Pixbuf;
        }

        protected void OnBtnNextClicked(object sender, System.EventArgs e)
        {
            if (state != stateMAX)
            {
                state++;
            }
            Fsm();
        }

        protected void OnBtnFinishClicked(object sender, System.EventArgs e)
        {
            this.Hide();
        }

        protected void OnBtnPrevClicked(object sender, System.EventArgs e)
        {
            if (state != 0)
            {
                state--;
            }
            Fsm();
        }

        #region Finite state machine for help showing
        void Fsm()
        { //change content of TextView according to value of "state" variable
            switch (state)
            {
                case 0:
                    textview3.Buffer.Text = @"How to put Kindle to fastboot mode:

-  Connect Kindle to USB
-  Open Kindle root directory
-  Rename file called 'DONT_HALT_ON_REPAIR' to
    'ENABLE_DIAGS'

To continue click 'Next step'.";

                    if (Regex.IsMatch(os, "Windows"))
                    {
                        pic.File = Directory.GetCurrentDirectory() + "\\lib\\pics\\1.jpg";
                    }
                    else
                    {
                        pic.File = Directory.GetCurrentDirectory() + "/lib/pics/1.jpg";
                    }
                    image2.Pixbuf = pic.Pixbuf;
                    break;
                case 1:
                    textview3.Buffer.Text = @"-  Disconnect Kindle from USB
-  Get the Kindle
-  Open 'Menu'
-  Go to 'Settings'
-  Open 'Menu' again
-  Select 'Restart'";
                    if (Regex.IsMatch(os, "Windows"))
                    {
                        pic.File = Directory.GetCurrentDirectory() + "\\lib\\pics\\2.jpg";
                    }
                    else
                    {
                        pic.File = Directory.GetCurrentDirectory() + "/lib/pics/2.jpg";
                    }
                    image2.Pixbuf = pic.Pixbuf;
                    break;
                case 2:
                    textview3.Buffer.Text = @"-  Select: 'D) Exit, Reboot or Disable Diags'
-  Then: 'F) Fastboot Bundle Install'

-  Press left arrow on fiveway button.
   (On Kindle touch select 'Q)' option)";

                    if (Regex.IsMatch(os, "Windows"))
                    {
                        pic.File = Directory.GetCurrentDirectory() + "\\lib\\pics\\3.jpg";
                    }
                    else
                    {
                        pic.File = Directory.GetCurrentDirectory() + "/lib/pics/3.jpg";
                    }
                    image2.Pixbuf = pic.Pixbuf;
                    break;
                case 3:
                    textview3.Buffer.Text = @"Now Kindle should be in fastboot mode.

If you have some troubles with this procedure,
follow this link for alternative way
(so far for Windows only):
http://bit.ly/LQDCk0

If you are running Windows, click 'Next step' button.
Otherwise you can close this window and start work
with fastboot.";
                    if (Regex.IsMatch(os, "Windows"))
                    {
                        frame1.Visible = false;
                        BtnK4Driver.Visible = false;
                        BtnKTDriver.Visible = false;
                    }
                    pic.File = "";
                    image2.Pixbuf = null;
                    break;
                case 4:
                    textview3.Buffer.Text = @"On Windows is needed to
install USB driver.

-  Be sure you have Kindle connected to computer 
   in fastboot mode and then click 
   'Install driver for K4' button or
   'Install driver for KT' button,
   according to your Kindle type.

After this, Kindle drivers should install automatically.";
                    if (Regex.IsMatch(os, "Windows"))
                    {
                        frame1.Visible = true;
                        BtnK4Driver.Visible = true;
                        BtnKTDriver.Visible = true;
                    }
                    BtnNext.Label = "Next step";
                    pic.File = "";
                    image2.Pixbuf = null;
                    break;
                case 5:
                    textview3.Buffer.Text = @"Now you can close this window and start work 
with fastboot.";
                    if (Regex.IsMatch(os, "Windows"))
                    {
                        pic.File = Directory.GetCurrentDirectory() + "\\lib\\pics\\5.jpg";
                    }
                    else
                    {
                        pic.File = Directory.GetCurrentDirectory() + "/lib/pics/5.jpg";
                    }
                    image2.Pixbuf = pic.Pixbuf;
                    break;
            }
        }

        #endregion

        protected void OnDeleteEvent(object o, Gtk.DeleteEventArgs args)
        {
            args.RetVal = true;
            this.Hide();
        }

        protected void ErrorHide(object sender, System.EventArgs e)
        {
            win.ErrorD.Hide();
            this.Sensitive = true;
        }

        #region Windows-only buttons

        void ShowError(int ret)
        {
            switch (ret)
            {
                case 0:
                    win.ErrorD.SetErrorText("USB driver has been installed successfully.");
                    win.ErrorD.SetButtonEvent(ErrorHide);
                    this.Sensitive = false;
                    //win.ErrorD.ActivateFocus ();
                    win.ErrorD.Show();
                    break;
                default:
                    win.ErrorD.SetErrorText("USB driver install failed. See instruction box for more information.");
                    win.ErrorD.SetButtonEvent(ErrorHide);
                    this.Sensitive = false;
                    //win.ErrorD.ActivateFocus ();
                    win.ErrorD.Show();
                    textview3.Buffer.Text = @"
If you have 64-bit Windows, please install
drivers manually from:
" + Directory.GetCurrentDirectory() + "\\lib\\drivers\\Kx\\Kindle.inf";
                    break;
            }
        }
        protected void OnBtnDriverK4Clicked(object sender, System.EventArgs e)
        {
            //int usb_install_driver_np(const char *inf_file); //function from lib-usb for driver install
            int ret = usb_install_driver_np(Directory.GetCurrentDirectory() + "\\lib\\drivers\\K4\\Kindle.inf");

            ShowError(ret);
        }

        protected void OnBtnDriverKTClicked(object sender, System.EventArgs e)
        {
            //int usb_install_driver_np(const char *inf_file); //function from lib-usb for driver install could be used too
            int ret = usb_install_driver_np(Directory.GetCurrentDirectory() + "\\lib\\drivers\\KT\\Kindle.inf");
            ShowError(ret);
        }

//        protected void OnBtnUSBClicked(object sender, System.EventArgs e)
//        {
//            //copy libusb dll - obsolete
//            string sys = Environment.GetFolderPath(Environment.SpecialFolder.System);
//            try
//            {
//                File.Copy(Directory.GetCurrentDirectory() + "\\lib\\libusb\\libusb0.dll", sys + "\\libusb0.dll", true);
//            }
//            catch (Exception ex)
//            {
//                textview3.Buffer.Text = "Problem when copying USB library: " + ex.ToString() + @"
//to continue press 'Next step' button.";
//            }
//        }
        #endregion
    }
}

