using System;
using Gtk;
using System.Diagnostics;
using System.IO;
using System.Runtime.InteropServices;
using System.Threading;
using System.Text.RegularExpressions;
using System.Collections;
using FastbootGUI;

namespace FastbootGUI
{
	public partial class MainWindow: Gtk.Window
	{		
		public const int MAX_FILESIZE = 67108864;
		public ErrorDialog ErrorD;
		PasswordDialog Password;
		AboutWindow About;
		HelpWindow  Help;
		public System.OperatingSystem osInfo = System.Environment.OSVersion;
		public static string pass = "";

		//public Logger LogFile;
		
		public MainWindow (): base (Gtk.WindowType.Toplevel)
		{
			Build ();		
			Tasks.AppendColumn ("Type of task", new Gtk.CellRendererText (), "text", 0);
			Tasks.AppendColumn ("Options", new Gtk.CellRendererText (), "text", 1);
			Gtk.ListStore ListStore = new Gtk.ListStore (typeof(string), typeof(string));
			Tasks.Model = ListStore;
			
			Password = new PasswordDialog (this);
			ErrorD = new ErrorDialog (this);
			About = new AboutWindow ();
			Help = new HelpWindow (osInfo.ToString (), this);
			this.Hide ();
			
			if (!File.Exists ("lib/fastboot")) {
				ErrorD.SetErrorText ("Fastboot binary hasn't been found.");
				ErrorD.SetButtonEvent (ExitAll);
				ErrorD.ActivateFocus ();
				ErrorD.Show ();
			} else {
				if (Regex.IsMatch (osInfo.ToString (), "^Unix")) {
					if (!File.Exists ("lib/run.sh")) {
						ErrorD.SetErrorText ("File \"run.sh\" hasn't been found.");
						ErrorD.SetButtonEvent (ExitAll);
						ErrorD.ActivateFocus ();
						ErrorD.Show ();
					} else {
						this.Show ();
					}
				} else {
					this.Show ();
				}
			}
			
//			if (Regex.IsMatch (osInfo.ToString (), "Windows")) {
//				//under Windows install libusb automatically, if not installed - not anymore
			//-- libusb is installed with drivers
//				//libusb
//				string sys = Environment.GetFolderPath (Environment.SpecialFolder.System);
//				if (!File.Exists (Directory.GetCurrentDirectory () + "\\lib\\libusb_installed")) {
//					File.Copy (Directory.GetCurrentDirectory () + "\\lib\\libusb\\libusb0.dll", sys + "\\libusb0.dll", true);
//					File.Create (Directory.GetCurrentDirectory () + "\\lib\\libusb_installed");
//				}
//			}
			
			//LogFile = new Logger ();
		}
		
		protected void NoCloseEvent (object sender, DeleteEventArgs a)
		{
			a.RetVal = true;
			Application.Run ();
		}
	
		protected void OnDeleteEvent (object sender, DeleteEventArgs a)
		{
			ErrorD.Destroy ();
			Password.Destroy ();
			About.Destroy ();
			//LogFile.closeFile ();
			this.Destroy ();
			Application.Quit ();
		}
	
		int AddTask (string TaskType, string Var = "", string File = "")
		{
			((Gtk.ListStore)Tasks.Model).AppendValues (TaskType, Var + " " + File);			
			return 0;
		}
		
		public void ExitAll (object sender, EventArgs a)
		{
			Environment.Exit (1);
		}

		protected void OnBtnAddGetVarClicked (object sender, System.EventArgs e)
		{
			AddTask ("getvar", combGetVarMode.ActiveText);
		}

		protected void OnBtnAddSetVarClicked (object sender, System.EventArgs e)
		{
			AddTask ("setvar", combSetVarMode.ActiveText, combSetVarProp.ActiveText);
		}

		protected void OnBtnAddFlashClicked (object sender, System.EventArgs e)
		{
			if (String.Compare (fileFlashChooser.Filename, "") > 0) {
				FileInfo info = new FileInfo (fileFlashChooser.Filename);
					
				if (info.Length > MAX_FILESIZE) {
					ErrorD.SetErrorText ("Too big file.");
					ErrorD.ActivateFocus ();
					ErrorD.Show ();
					return;
				} else {
					AddTask ("flash", combFlashMode.ActiveText, fileFlashChooser.Filename);
				}
			} else {
				ErrorD.SetErrorText ("First select a file to flash.");
				ErrorD.Show ();
			}
		}
	
		protected void OnBtnDeleteItemClicked (object sender, System.EventArgs e)
		{
			TreeSelection selection = (Tasks as TreeView).Selection;
			TreeModel model;
			TreeIter iter;

			selection.GetSelected (out model, out iter);
			string s = Tasks.Model.GetStringFromIter (iter);
			((Gtk.ListStore)Tasks.Model).Remove (ref iter);
			
			Tasks.Model.GetIterFromString (out iter, s);
			selection.SelectIter (iter);
		}

		protected void OnFileFlashChooserSelectionChanged (object sender, System.EventArgs e)
		{
			//entryFlashFilename.Text = fileFlashChooser.Filename;
		}

		protected void OnBtnAddRebootClicked (object sender, System.EventArgs e)
		{
			AddTask ("reboot");
		}
	
		public struct ThreadParams
		{
			public string task;
			public string opt;
		}

		protected void OnBtnExecAllClicked (object sender, System.EventArgs e)
		{
			TreeIter iter;
			
			try {
				((Gtk.ListStore)Tasks.Model).GetIterFirst (out iter); //test
				if (iter.Stamp == 0) {
					return;
				}
			} catch {
				return;
			}
			
			if (Regex.IsMatch (osInfo.ToString (), "^Unix") && (String.Compare (pass, "") == 0)) {
				Password.ShowAll ();
				//LogFile.append ("Password dialog showed." + System.Environment.NewLine);
			} else {
				//LogFile.append ("Thread started." + System.Environment.NewLine);
				Thread ProcessRunner = new Thread (new ThreadStart (this.StartProcess));
				ProcessRunner.Start ();
			}
		}
		
		public void StartProcessEx ()
		{
			//LogFile.append ("Thread started." + System.Environment.NewLine);
			Thread ProcessRunner = new Thread (new ThreadStart (this.StartProcess));
			ProcessRunner.Start ();
		}
		
		public void StartPulser ()
		{
			//LogFile.append ("Pulser started." + System.Environment.NewLine);
			while (true) {
				try {
					while (Gtk.Application.EventsPending())
						;
					Gtk.Application.Invoke (delegate {
						progressbar.Pulse ();
					});
					Thread.Sleep (100);
				} catch {
					while (Gtk.Application.EventsPending())
						;
					Gtk.Application.Invoke (delegate {
						progressbar.Fraction = 0;
					});
					break;
				}
			}
		}
		
//		public void StartProcess ()
//		{
//			Thread.Sleep (200);
//		}
		
		public void StartProcess ()
		{
			// Make progress bar pulsing
			Thread progressbarPulser = new Thread (new ThreadStart (this.StartPulser));
			progressbarPulser.Start ();
		
			TreeIter iter;
			TextIter mIter = Log.Buffer.EndIter;
			
			//Block all buttons and help
			btnExecAll.Sensitive = false;
			btnDeleteItem.Sensitive = false;
			btnAddSetVar.Sensitive = false;
			btnAddGetVar.Sensitive = false;
			btnAddReboot.Sensitive = false;
			btnAddFlash.Sensitive = false;
			menubar1.Sensitive = false;
			
			//Disallow to close main window
			this.DeleteEvent += NoCloseEvent;
			this.DeleteEvent -= OnDeleteEvent;
			
			//Thread lock
			object dispLock = new object ();
			
			//Some another synchronization gizmo (which actually works)
			ManualResetEvent reset = new ManualResetEvent (false);
			
			//New process
			Process proc = new Process ();
		
			//Process Start information
			proc.StartInfo.FileName = System.IO.Directory.GetCurrentDirectory () + "\\lib\\fastboot"; // default for Windows
			proc.StartInfo.FileName = char.ToUpper (proc.StartInfo.FileName [0]) + proc.StartInfo.FileName.Substring (1); //uppercase first letter
			
			proc.StartInfo.UseShellExecute = false;
			proc.StartInfo.RedirectStandardError = true;
			proc.StartInfo.RedirectStandardInput = false;
			proc.StartInfo.RedirectStandardOutput = false;
			proc.StartInfo.CreateNoWindow = true;
			proc.StartInfo.WorkingDirectory = System.IO.Directory.GetCurrentDirectory ();
			proc.StartInfo.WorkingDirectory = char.ToUpper (proc.StartInfo.WorkingDirectory [0]) + proc.StartInfo.WorkingDirectory.Substring (1); //uppercase first letter
			
			
			if (String.Compare (pass, "") != 0) { // Linux (and hopefully Mac OS) - because password was inserted
				proc.StartInfo.FileName = System.IO.Directory.GetCurrentDirectory () + "/lib/run.sh";
				//processStartInfo.Arguments = pass + " ";
				//pass = ""; //in future might be "forget password" feature implemented, with settings
			}

			int i = 0;
		
			((Gtk.ListStore)Tasks.Model).GetIterFromString (out iter, i.ToString ()); //row
			
			string task;
			string opt;
		
			try {
				task = ((Gtk.ListStore)Tasks.Model).GetValue (iter, 0).ToString (); //col
				opt = ((Gtk.ListStore)Tasks.Model).GetValue (iter, 1).ToString (); //col
			} catch {
				return;
			}
		
			while (String.Compare(task, "") != 0) {  // there is needed to walk through tasks (fastboot operations) and run it one by one
				
				proc.StartInfo.Arguments = pass + " ";
				proc.StartInfo.Arguments = proc.StartInfo.Arguments + task + " " + opt;
				
				//LogFile.append ("Process will start with arguments:" + processStartInfo.Arguments + System.Environment.NewLine);

				//Start process
				bool processStarted = false;
				try {
					processStarted = proc.Start ();
				} catch (Exception ex) {
					ex.ToString ();
				}
				
				if (processStarted) {				
					//Display the result
					string displayText;					
					proc.WaitForExit ();
					
					lock (dispLock) {
						displayText = proc.StandardError.ReadLine ();
					}
					
					try { //because of ArgumentNullException in Windows when no Kindle is connected
						if (Regex.IsMatch (displayText, "< waiting for device >")) {
							while (Gtk.Application.EventsPending())
								;
							Gtk.Application.Invoke (delegate {
								lock (dispLock) {
									Log.Buffer.Insert (ref mIter, "No Kindle in fastboot has been found." +
							System.Environment.NewLine + "Please refer to Help." + System.Environment.NewLine);
									reset.Set ();
								}
							});
							reset.WaitOne ();
							reset.Reset ();
							break;
						} else {
							if (Regex.IsMatch (displayText, "Sorry")) {
								displayText = "Wrong root password has been inserted.";
							
								pass = "";
								while (Gtk.Application.EventsPending())
									;
								Gtk.Application.Invoke (delegate {
									lock (dispLock) {
										Log.Buffer.Insert (ref mIter, displayText + System.Environment.NewLine);
										reset.Set ();
									}
								});
								reset.WaitOne ();
								reset.Reset ();
								break;
							} else {
								if (Regex.IsMatch (task, "getvar")) {
									opt = Regex.Replace (opt, " ", "");
									displayText = Regex.Replace (displayText, "^.*" + opt + ":", "");
									opt = char.ToUpper (opt [0]) + opt.Substring (1);
								
									Gtk.Application.Invoke (delegate {
										lock (dispLock) {
											Log.Buffer.Insert (ref mIter, opt + ": " + displayText + System.Environment.NewLine);
											reset.Set ();
										}
									});
									reset.WaitOne ();
									reset.Reset ();
								
								}
								if (Regex.IsMatch (task, "setvar")) {
									displayText = Regex.Replace (displayText, "^.*setting ", "");
									displayText = Regex.Replace (displayText, ":", "");
									displayText = Regex.Replace (displayText, "'", "");
							
									displayText = char.ToUpper (displayText [0]) + displayText.Substring (1);
									displayText = Regex.Replace (displayText, "to", "has been set to");
									displayText = Regex.Replace (displayText, "\\.\\.\\.", "");
								
									while (Gtk.Application.EventsPending())
										;
									Gtk.Application.Invoke (delegate {
										lock (dispLock) {
											Log.Buffer.Insert (ref mIter, displayText + System.Environment.NewLine);
											reset.Set ();
										}
									});
									reset.WaitOne ();
									reset.Reset ();								
								}
								if (Regex.IsMatch (task, "flash")) {
									displayText = Regex.Replace (displayText, "^.*downloading", "Flashing");
									displayText = Regex.Replace (displayText, "'", "");
									displayText = Regex.Replace (displayText, "\\.\\.\\.", "");
													
									while (Gtk.Application.EventsPending())
										;
									Gtk.Application.Invoke (delegate {
										lock (dispLock) {
											Log.Buffer.Insert (ref mIter, displayText + System.Environment.NewLine);
											reset.Set ();
										}
									});
									reset.WaitOne ();
									reset.Reset ();
								
									displayText = proc.StandardError.ReadLine ();
								
								
									displayText = Regex.Replace (displayText, "OKAY", "Flashing finished in:");
									displayText = Regex.Replace (displayText, "\\[", "");
									displayText = Regex.Replace (displayText, "\\]", "");
								
									while (Gtk.Application.EventsPending())
										;
									Gtk.Application.Invoke (delegate {
										lock (dispLock) {
											Log.Buffer.Insert (ref mIter, displayText + System.Environment.NewLine);
											reset.Set ();
										}
									});
									reset.WaitOne ();
									reset.Reset ();
								}
								if (Regex.IsMatch (task, "reboot")) {
									displayText = Regex.Replace (displayText, "^.*rebooting", "Rebooting");
									displayText = Regex.Replace (displayText, ":", "");
									displayText = Regex.Replace (displayText, "'", "");
							
									displayText = char.ToUpper (displayText [0]) + displayText.Substring (1);
									displayText = Regex.Replace (displayText, "to", "has been set to");
									displayText = Regex.Replace (displayText, "\\.\\.\\.", "");
								
									while (Gtk.Application.EventsPending())
										;
									Gtk.Application.Invoke (delegate {
										lock (dispLock) {
											Log.Buffer.Insert (ref mIter, displayText + System.Environment.NewLine);
											reset.Set ();
										}
									});
									reset.WaitOne ();
									reset.Reset ();
								}
						
								while (Gtk.Application.EventsPending())
									;
								Gtk.Application.Invoke (delegate {
									lock (dispLock) {
										((Gtk.ListStore)Tasks.Model).Remove (ref iter);
										reset.Set ();
									}
								});
								reset.WaitOne ();
								reset.Reset ();
							}
						}
					} catch {
						while (Gtk.Application.EventsPending())
							;
						Gtk.Application.Invoke (delegate {
							lock (dispLock) {
								Log.Buffer.Insert (ref mIter, "No Kindle in fastboot has been found." +
							System.Environment.NewLine + "Please refer to Help." + System.Environment.NewLine);
								reset.Set ();
							}
						});
						reset.WaitOne ();
						reset.Reset ();
						break;
					}
				}
			
				((Gtk.ListStore)Tasks.Model).GetIterFromString (out iter, i.ToString ()); //radek
			
				proc.StartInfo.Arguments = pass + " ";
				if (iter.Stamp != 0) {
					while (Gtk.Application.EventsPending())
						;
					Gtk.Application.Invoke (delegate {
						lock (dispLock) {
							try {
								task = ((Gtk.ListStore)Tasks.Model).GetValue (iter, 0).ToString (); //sloupec
								opt = ((Gtk.ListStore)Tasks.Model).GetValue (iter, 1).ToString (); //sloupec
								reset.Set ();
							} catch (Exception ex) {
								ex.ToString ();
							}
						}
					});
					reset.WaitOne ();
					reset.Reset ();
				} else {
					task = "";
				}
			}
			progressbarPulser.Abort ();
			while (progressbarPulser.IsAlive)
				;
			
			while (Gtk.Application.EventsPending())
				;
			Gtk.Application.Invoke (delegate {
				lock (dispLock) {
					//Unlock all buttons and help
					progressbar.Fraction = 0;
					progressbar.Sensitive = false;
					btnExecAll.Sensitive = true;
					btnDeleteItem.Sensitive = true;
					btnAddSetVar.Sensitive = true;
					btnAddGetVar.Sensitive = true;
					btnAddReboot.Sensitive = true;
					btnAddFlash.Sensitive = true;
					menubar1.Sensitive = true;
					reset.Set ();
				}
			});
			reset.WaitOne ();
			reset.Reset ();
			
			//Allow to close main window
			this.DeleteEvent -= NoCloseEvent;
			this.DeleteEvent += OnDeleteEvent;
		}

		protected void OnAboutActionActivated (object sender, System.EventArgs e)
		{
			About.Show ();
		}

		protected void OnContentsActionActivated (object sender, System.EventArgs e)
		{
			Help.Show ();
		}
	}
}